from blipy.Types import Data, Method, Response
from requests import post

BLIP_COMMANDS = 'https://msging.net/commands'
BLIP_TAKE_MAX = 100

class Lime():
	def __init__(self, authorization):
		self.header = {
			'Authorization': f'Key {authorization}',
			'Content-Type': 'application/json'
		}

	def __sendCommand(self, method, uri, toId, resourceType = None, resource = None):
		data = Data.new(method, uri, toId, resourceType, resource)
		return Response(post(BLIP_COMMANDS, data=data, headers=self.header))

	def Get(self, uri, toId):
		return self.__sendCommand(Method.Get, uri, toId)

	def Set(self, uri, toId, resourceType, resource):
		return self.__sendCommand(Method.Set, uri, toId, resourceType, resource)

	def Delete(self, uri, toId):
		return self.__sendCommand(Method.Delete, uri, toId)

	def GetAll(self, uri, toId):
		updateUri = lambda uri, skip: f'{uri}?$skip={skip}&$take={BLIP_TAKE_MAX}'
		response = self.__sendCommand(Method.Get, updateUri(uri, 0), toId)
		resource = response.Resource
		skip = resource.Total
		getMore = resource.Total == BLIP_TAKE_MAX
		while getMore:
			moreResource = self.__sendCommand(Method.Get, updateUri(uri,skip), toId).Resource
			resource.addItems(moreResource)
			skip += moreResource.Total
			getMore = moreResource.Total == BLIP_TAKE_MAX
		return response

	def SetAll(self, uri, toId, resourceType, resources):
		completed = []
		for resource in resources:
			response = self.__sendCommand(Method.Set, uri, toId, resourceType, resource)
			if response.Success:
				contentId = response.Resource.Id
				completed.append(contentId)
			else:
				self.DeleteAll(uri, toId, completed)
				raise Exception('Error occured while trying to set all resource')
		return completed

	def DeleteAll(self, uri, toId, ids):
		uriWithId = lambda uri, uriId: f'{uri}/{uriId}'
		responses = []
		for uriId in ids:
			responses.append(self.__sendCommand(Method.Delete, uriWithId(uri,uriId), toId))
		return responses



from uuid import uuid1
from json import dumps

class Data:
	@staticmethod
	def new(method, uri, toId, resourceType = 'None', resource = None):
		data = {
			'id': str(uuid1()),
			'uri': uri,
			'to': f'{toId}.msging.net' if '@' in toId else f'{toId}@msging.net',
			'method': method
		}
		if resourceType and resource:
			data['type'] = resourceType
			data['resource'] = resource
		return dumps(data)



from blipy.Http import Lime

class Postmaster(Lime):
	def __init__(self, authorization, name):
		super().__init__(authorization)
		self.identity = f'postmaster@{name}'

	def Get(self, uri):
		return super().Get(uri, self.identity)
	
	def Set(self, uri, resourceType, resource):
		return super().Set(uri, self.identity, resourceType, resource)

	def Delete(self, uri):
		return super().Delete(uri, self.identity)
	
	def GetAll(self, uri):
		return super().GetAll(uri, self.identity)
	
	def SetAll(self, uri, resourceType, resources):
		return super().SetAll(uri, self.identity, resourceType, resources)
	
	def DeleteAll(self, uri, ids):
		return super().DeleteAll(uri, self.identity, ids)



from blipy.Types import Seriable

keyValueOrDefault = lambda source, key, default: source[key] if key in source else default

class Resource(Seriable):
	def __init__(self, rawResource):
		super().__init__(rawResource)
		self.Total = keyValueOrDefault(rawResource, 'total', 0)
		self.Items = keyValueOrDefault(rawResource, 'items', [])
		self.Id = keyValueOrDefault(rawResource, 'id', None)

	def addItems(self, resource):
		self.Total += resource.Total
		self.Items += resource.Items

class Response(Seriable):
	def __init__(self, rawResponse):
		response = rawResponse.json()
		super().__init__(response)
		self.Status = keyValueOrDefault(response, 'status', None)
		self.Resource = Resource(self.Resource)
		self.Success = self.Status == 'success'



from blipy.Http import Postmaster

class AIPostmaster(Postmaster):
	def __init__(self, authorization):
		super().__init__(authorization, 'ai')
	
	def getIntent(self, intentId):
		return super().Get(f'/intentions/{intentId}?deep=true')

	def getIntents(self):
		return super().GetAll('/intentions')

	def getEntity(self, entityId):
		return super().Get(f'/entities/{entityId}')



class URI:
	@staticmethod
	def new(baseuri, params = None):
		checkForParams = baseuri.split('?')
		base = checkForParams[0]
		if params:
			keyEqualsValue = lambda key: f'{key}={params[key]}'
			base += '?' + '&'.join(map(keyEqualsValue, params))
		if len(checkForParams) == 2:
			base += ('' if params else '?') + '&' + checkForParams[1]
		return base
